@@ -11,16 +11,42 @@ import types
 import inspect
 import functools
 from typing import Callable, Any
-from dataclasses import Field, _FIELD, MISSING, field, InitVar, _is_classvar, _FIELD_CLASSVAR, \
+from dataclasses import Field, _FIELD, MISSING, field, fields, InitVar, _is_classvar, _FIELD_CLASSVAR, \
     _is_type, _is_initvar, _FIELD_INITVAR, _PARAMS, _DataclassParams, _FIELDS, _POST_INIT_NAME, _set_new_attribute, \
     _repr_fn, _tuple_str, _cmp_fn, _frozen_get_del_attr, _hash_action, _init_fn
 
 
-__all__ = ['field_property', 'dataclass', '_get_field', '_process_class']
+__all__ = ['field_property', 'get_return_type', 'dataclass', '_get_field', '_process_class']
+
+
+def get_return_type(default: Any = MISSING, default_factory: Callable[[], Any] = MISSING):
+    """Find the return type for the default value or default_factory.
+
+    Args:
+        default (object)[MISSING]: is the default value of the field.
+        default_factory (callable/function)[MISSING]: is a 0-argument function called to initialize a field's value.
+
+    Returns:
+        return_type (type/object)[MISSING]: The type of the default or MISSING if not found.
+    """
+    # Fields must have an annotation
+    return_type = inspect.signature(default_factory).return_annotation
+    if return_type == inspect.Signature.empty:
+        return_type = MISSING
+        if default != MISSING:
+            return_type = type(default)
+        elif default_factory != MISSING:
+            try:
+                return_type = type(default_factory())
+            except (ValueError, TypeError, Exception):
+                pass
+    return return_type
 
 
 class field_property(property):
 
+    get_return_type = staticmethod(get_return_type)
+
     def __init__(self,
                  fget: Callable[[Any], Any] = None,
                  fset: Callable[[Any, Any], None] = None,
@@ -31,8 +57,16 @@ class field_property(property):
 
         self.default_attr = default
         self.default_factory_attr = default_factory
+        self.name = None
         super().__init__(fget, fset, fdel, doc=doc)
 
+    def __set_name__(self, owner, name):
+        try:
+            if self.name is None:
+                self.name = name
+        except (AttributeError, Exception):
+            pass
+
     def getter(self, fget: Callable[[Any], Any]) -> 'field_property':
         return type(self)(fget, self.fset, self.fdel, self.__doc__,
                           default=self.default_attr, default_factory=self.default_factory_attr)
@@ -75,8 +109,8 @@ def dataclass(cls=None, *, init=True, repr=True, eq=True, order=False,
             cls.__annotations__ = {}
         for name, attr in cls.__dict__.items():
             if isinstance(attr, property) and name not in cls.__annotations__:
-                return_type = inspect.signature(attr.fget).return_annotation
-                if return_type != inspect.Signature.empty:
+                return_type = get_return_type(default_factory=attr.fget)
+                if return_type != MISSING:
                     cls.__annotations__[name] = return_type
 
         return _process_class(cls, init, repr, eq, order, unsafe_hash, frozen)
